#include once "vec2f.bi"
#include once "model.bi"
#include once "camera.bi"
 

extern levWidth as integer
extern levHeight as integer

enum BONUS_ENUM

QUICK_SHOT = 1
SHIELD = 2
BRAKE = 4
ARRAY = 8
ROTBOOST = 16
TRACTOR_BEAM = 32
RETICLE = 64
BIGBATT = 128
BIGPING = 256

end enum


enum PROJECTILE_ENUM

SHOT
BOMB

end enum


enum PARTICLE_ENUM

	EXPLOSION
	SHIELD
	THRUST
	PING

end enum


type base_struct
	
	as vec2f p, v, targV, dirVec, collideP
	
	as byte active
	
	as single rot, targRot, rotDelta
	
end type


type particle_struct extends base_struct
	
	as vec2f pb
	
	pType as PARTICLE_ENUM
	
	as single size, sizeDelta, sizeMax, lSpan, vDecay
	
	as rgb_struct colour, colDelta
	
	as FB.IMAGE ptr sprite
	
end type


type particle_system
	
	parts(any) as particle_struct
	
	declare sub render(byref cam as camera_struct)
	declare sub init( byref cnt as integer )
	declare sub update( byref tDelta as single, byref cam as camera_struct )
	declare sub spawn_shield(byref sp as vec2f, byref ep as vec2f, byref vel as vec2f)	
	declare sub spawn_explosion( byref posit as vec2f, byref mag as single, byref maxParts as integer = 19, byref partType as PARTICLE_ENUM, byref lifeSpan as single)
	declare sub spawn_ping( byref posit as vec2f, byref radDelta as single, byref ang as single, byref partType as PARTICLE_ENUM )
	declare sub spawn_thrust( byref posit as vec2f, byref v1 as vec2f,  byref v2 as vec2f, byref vel as vec2f, byref ang as single, byref sprite as FB.IMAGE ptr, byref lifeSpan as single)
	
end type


type entity_struct extends base_struct

	as byte special, hasSpecial, doDeath

	as single mass, vDelta, vMag, spawnWait

	model(any) as model_struct

	as rgb_struct colour
	
	as particle_system ptr prt
	
end type


type projectile_struct extends base_struct

	as PROJECTILE_ENUM pType

	as vec2f oP
	
	as single lSpan
	
end type


type bonus_struct extends entity_struct

	mType as BONUS_ENUM

end type


type bonus_system
	
	bonus(any) as bonus_struct
	
	declare sub init( byref cnt as integer, byref bTypes as BONUS_ENUM )
	declare sub update( byref tDelta as single )
	declare sub spawn()
	
end type


type player_struct extends entity_struct
	
	as integer pingActive, level, score, sCount, bCount, sMax, bMax, modifiers

	as integer shotCount, bombCount

	as single shotTimer, shotWait

	as single bombTimer, bombWait

	as single pingTimer, pingWait, pingRad, oPingRaD, pingRange
	
	as single shieldTime, reticleTime
	
	as vec2f pingOrigin

	as single thrust, fuel
	
	as single battery

	as projectile_struct proj(any)
	
	declare sub add_fuel( byref amt as single )
	
	declare sub subtract_fuel( byref amt as single )
	
	declare sub add_battery( byref amt as single )
	
	declare sub subtract_battery( byref amt as single )
	
	
	as single maxFuel = 100, maxBattery = 100
	
	as ushort retPat = &b0000000011111111
	
end type


declare sub update_player( pl as player_struct, Asteroids() as entity_struct, byref tDelta as single, byref cam as camera_struct  )
declare sub update_asteroids( Asteroids() as entity_struct, byref tDelta as single, byref levelUp as byte, byref cam as camera_struct )
