#include "vec2f.bi"



constructor vec2f()
end constructor


constructor vec2f(byref lhs as vec2f)

this.x = lhs.x
this.y = lhs.y

end constructor


constructor vec2f(byref x as integer, byref y as integer)

this.x = x
this.y = y

end constructor


constructor vec2f(byref x as single, byref y as single)

this.x = x
this.y = y

end constructor


function vec2f.dist( byref vb as vec2f ) as single

	dim as single d

	dim as single dx
	dim as single dy

	dx = this.x - vb.X
	dy = this.y - vb.Y

	d = sqr(dx^2+dy^2)

	if d = 0 then d = 1

	return  d

end function


function vec2f.magnitude( ) as single

	dim Mag as single = any
	mag = sqr( this.x * this.x + this.y * this.y )
	if mag = 0 then mag = 1
	return mag

end function


sub vec2f.normalize()

	this = this / this.magnitude()

end sub


function vec2f.perp( byref b as vec2f ) as vec2f

	dim as single vLen
	dim as vec2f d

	d.x = b.x - this.x
	d.y = b.y - this.y

	vLen = b.dist(this)

	return type( d.y  / vLen, -d.x / vLen )

end function


function lines_intersect( byref ipoint as vec2f, byref va as vec2f, byref vb as vec2f, byref vc as vec2f, byref vd as vec2f ) as integer

	dim as vec2f v1 = vb - va
	dim as vec2f v2 = vd - vc
	dim as vec2f v3 = vc - va
	dim as single perp = (v1.x * v2.y) - (v1.y * v2.x)

	if perp<>0 then
		dim as single d1 = ((v3.x * v2.y) - (v3.y * v2.x)) / perp
		if  d1<=1 and d1>=0 then
			dim as single d2 = ((v3.x * v1.y) - (v3.y * v1.x)) / perp
			if d2<=1 and d2>=0 then
				dim as single m1, m2, c1, c2, recip

				if v1.x <> 0 then
					m1 = v1.y / v1.x
				else
					m1 = 1
				end if

				if v2.x <> 0 then
					m2 = v2.y / v2.x
				else
					m2 = 1
				end if

				c1 = va.y - (m1*va.x)
				c2 = vc.y - (m2*vc.x)

				recip = 1 / -(m1 - m2)

				ipoint.x = -(c2-c1) * recip
				ipoint.y = ((m2*c1) - (m1*c2)) * recip

				return true
			end if
		end if
	end if

	return false

end function


function vec2f.cDist( byref v2 as vec2f ) as single
	
	static as integer ti
	static as single tDist, xHalf
	
		tDist = (x*v2.x + y*v2.y)
		xHalf = 0.5f*tDist
		ti = *cast( integer ptr, @tDist )
		ti = &h5f3759df - (ti shr 1)
		tDist = *cast( single ptr, @ti )
		tDist = tDist*(1.5f - xHalf*tDist*tDist)
	
	return tDist
	
end function							



operator + ( byval lhs as vec2f, byval rhs as vec2f ) as vec2f
	return type<vec2f>( lhs.x + rhs.x, lhs.y + rhs.y )
end operator

operator + ( byval lhs as vec2f, byval rhs as single ) as vec2f
	return type<vec2f>( lhs.x + rhs, lhs.y + rhs )
end operator

operator - ( byval lhs as vec2f, byval rhs as vec2f ) as vec2f
	return type<vec2f>( lhs.x - rhs.x, lhs.y - rhs.y )
end operator

operator - ( byval lhs as vec2f, byval rhs as single ) as vec2f
	return type<vec2f>( lhs.x - rhs, lhs.y - rhs )
end operator

operator * ( byval lhs as vec2f, byval rhs as vec2f ) as vec2f
	return type<vec2f>( lhs.x * rhs.x, lhs.y * rhs.y )
end operator

operator * ( byval lhs as vec2f, byval rhs as single ) as vec2f
	return type<vec2f>( lhs.x * rhs, lhs.y * rhs )
end operator

operator / ( byval lhs as vec2f, byval rhs as vec2f ) as vec2f
	return type<vec2f>( lhs.x / rhs.x, lhs.y / rhs.y )
end operator

operator / ( byval lhs as vec2f, byval rhs as single ) as vec2f
	return type<vec2f>( lhs.x / rhs, lhs.y / rhs )
end operator

operator / ( byval lhs as vec2f, byval rhs as integer ) as vec2f
	return type<vec2f>( lhs.x / rhs, lhs.y / rhs )
end operator
